//----------------------------------------------------------------------------
//
// LFSR functions with x86 optimizations, by Scott Duplichan
// tested with Microsoft and mingw C compilers
//
//----------------------------------------------------------------------------
#include "project.h"
//----------------------------------------------------------------------------
// bit shift left a 64-bit value using general purpose registers
//          uint64_t *data - data to shift
//          int count      - number of bits to shift
// return:  uint64_t       - carry out bit(s)

static uint64_t bitShiftLeft64Gpr (uint64_t *data, int count)
   {
   uint64_t carryOut = *data >> (64 - count);
   *data <<= count;
   return carryOut;
   }

//----------------------------------------------------------------------------
// bit shift left a 128-bit value using general purpose registers
//          uint64_t *data - data to shift, ls part stored first 
//          int count      - number of bits to shift
// return:  uint64_t       - carry out bit(s)

static uint64_t bitShiftLeft128gpr (uint64_t *data, int count)
   {
   uint64_t carryOut0, carryOut1;

   carryOut0 = bitShiftLeft64Gpr (&data [0], count);
   carryOut1 = bitShiftLeft64Gpr (&data [1], count);
   data [1] |= carryOut0;
   return carryOut1;
   }

//----------------------------------------------------------------------------
// bit shift left a 256-bit value using general purpose registers
//          uint64_t *data - data to shift, ls part stored first 
//          int count      - number of bits to shift
// return:  uint64_t       - carry out bit(s)

static uint64_t bitShiftLeft256gpr (uint64_t *data, int count)
   {
   uint64_t carryOut0, carryOut1;

   carryOut0 = bitShiftLeft128gpr (&data [0], count);
   carryOut1 = bitShiftLeft128gpr (&data [2], count);
   data [2] |= carryOut0;
   return carryOut1;
   }

//----------------------------------------------------------------------------
// bit shift left a 512-bit value using general purpose registers
//          uint64_t *data - data to shift, ls part stored first 
//          int count      - number of bits to shift
// return:  uint64_t       - carry out bit(s)

static uint64_t bitShiftLeft512gpr (uint64_t *data, int count)
   {
   uint64_t carryOut0, carryOut1;

   carryOut0 = bitShiftLeft256gpr (&data [0], count);
   carryOut1 = bitShiftLeft256gpr (&data [4], count);
   data [4] |= carryOut0;
   return carryOut1;
   }

//----------------------------------------------------------------------------
// bit shift left a 1024-bit value using general purpose registers
//          uint64_t *data - data to shift, ls part stored first 
//          int count      - number of bits to shift
// return:  uint64_t       - carry out bit(s)

static uint64_t bitShiftLeft1024gpr (uint64_t *data, int count)
   {
   uint64_t carryOut0, carryOut1;

   carryOut0 = bitShiftLeft512gpr (&data [0], count);
   carryOut1 = bitShiftLeft512gpr (&data [8], count);
   data [8] |= carryOut0;
   return carryOut1;
   }

//----------------------------------------------------------------------------
// bit shift left a 2048-bit value using general purpose registers
//          uint64_t *data - data to shift, ls part stored first 
//          int count      - number of bits to shift
// return:  uint64_t       - carry out bit(s)

static uint64_t bitShiftLeft2048gpr (uint64_t *data, int count)
   {
   uint64_t carryOut0, carryOut1;

   carryOut0 = bitShiftLeft1024gpr (&data [ 0], count);
   carryOut1 = bitShiftLeft1024gpr (&data [16], count);
   data [16] |= carryOut0;
   return carryOut1;
   }

//----------------------------------------------------------------------------
// bit shift left a 4096-bit value using general purpose registers
//          uint64_t *data - data to shift, ls part stored first 
//          int count      - number of bits to shift
// return:  uint64_t       - carry out bit(s)

static uint64_t bitShiftLeft4096gpr (uint64_t *data, int count)
   {
   uint64_t carryOut0, carryOut1;

   carryOut0 = bitShiftLeft2048gpr (&data [ 0], count);
   carryOut1 = bitShiftLeft2048gpr (&data [32], count);
   data [32] |= carryOut0;
   return carryOut1;
   }

//----------------------------------------------------------------------------
//
// lfsr32gpr - left shift galois type lfsr for 32-bit data
//

static uint32_t lfsr32gpr (uint32_t data, uint32_t mask)
   {
   uint32_t carryOut = data >> 31;
   uint32_t maskOrZ = -carryOut; 
   return (data << 1) ^ (maskOrZ & mask);
   }

//----------------------------------------------------------------------------
//
// lfsr64gpr - left shift galois type lfsr for 64-bit data, general purpose register implementation
//
static uint64_t lfsr64gpr (uint64_t data, uint64_t mask)
   {
   uint64_t carryOut = data >> 63;
   uint64_t maskOrZ = -carryOut; 
   return (data << 1) ^ (maskOrZ & mask);
   }

//----------------------------------------------------------------------------
//
// lfsr128gpr - left shift galois type lfsr for 128-bit data, general purpose register implementation
//
static void lfsr128gpr (uint64_t *data, uint64_t *mask)
   {
   uint64_t maskOrZero [2];
   uint64_t carryOut = data [1] >> 63;
   uint64_t maskOrZ = -carryOut; 

   // zero the mask if data bit 127 is zero
   maskOrZero [0] = mask [0] & maskOrZ;
   maskOrZero [1] = mask [1] & maskOrZ;

   // shift the whole data left by one bit
   bitShiftLeft128gpr (data, 1);

   // xor in mask if bit 127 was set
   data [0] ^= maskOrZero [0];
   data [1] ^= maskOrZero [1];
   }

//----------------------------------------------------------------------------
//
// lfsr256gpr - left shift galois type lfsr for 256-bit data, general purpose register implementation
//
static void lfsr256gpr (uint64_t *data, uint64_t *mask)
   {
   uint64_t maskOrZero [4], maskOrZ;

   // expand data bit 255 into mask
   maskOrZ = (int64_t)data [3] >> 63;

   // zero the mask if data bit 255 is zero
   maskOrZero [0] = mask [0] & maskOrZ;
   maskOrZero [1] = mask [1] & maskOrZ;
   maskOrZero [2] = mask [2] & maskOrZ;
   maskOrZero [3] = mask [3] & maskOrZ;

   // shift the whole data left by one bit
   bitShiftLeft256gpr (data, 1);

   // xor in mask if bit 255 was set
   data [0] ^= maskOrZero [0];
   data [1] ^= maskOrZero [1];
   data [2] ^= maskOrZero [2];
   data [3] ^= maskOrZero [3];
   }

//----------------------------------------------------------------------------
//
// lfsr512gpr - left shift galois type lfsr for 512-bit data, general purpose register implementation
//
static void lfsr512gpr (uint64_t *data, uint64_t *mask)
   {
   uint64_t maskOrZero [8], maskOrZ;

   // expand data bit 511 into mask
   maskOrZ = (int64_t)data [7] >> 63;

   // zero the mask if data bit 511 is zero
   maskOrZero [0] = mask [0] & maskOrZ;
   maskOrZero [1] = mask [1] & maskOrZ;
   maskOrZero [2] = mask [2] & maskOrZ;
   maskOrZero [3] = mask [3] & maskOrZ;
   maskOrZero [4] = mask [4] & maskOrZ;
   maskOrZero [5] = mask [5] & maskOrZ;
   maskOrZero [6] = mask [6] & maskOrZ;
   maskOrZero [7] = mask [7] & maskOrZ;

   // shift the whole data left by one bit
   bitShiftLeft512gpr (data, 1);

   // xor in mask if bit 511 was set
   data [0] ^= maskOrZero [0];
   data [1] ^= maskOrZero [1];
   data [2] ^= maskOrZero [2];
   data [3] ^= maskOrZero [3];
   data [4] ^= maskOrZero [4];
   data [5] ^= maskOrZero [5];
   data [6] ^= maskOrZero [6];
   data [7] ^= maskOrZero [7];
   }

//----------------------------------------------------------------------------
//
// lfsr1024gpr - left shift galois type lfsr for 1024-bit data, general purpose register implementation
//
static void lfsr1024gpr (uint64_t *data, uint64_t *mask)
   {
   uint64_t maskOrZero [16], maskOrZ;

   // expand data bit 1023 into mask
   maskOrZ = (int64_t)data [15] >> 63;

   // zero the mask if data bit 1023 is zero
   maskOrZero [ 0] = mask [ 0] & maskOrZ;
   maskOrZero [ 1] = mask [ 1] & maskOrZ;
   maskOrZero [ 2] = mask [ 2] & maskOrZ;
   maskOrZero [ 3] = mask [ 3] & maskOrZ;
   maskOrZero [ 4] = mask [ 4] & maskOrZ;
   maskOrZero [ 5] = mask [ 5] & maskOrZ;
   maskOrZero [ 6] = mask [ 6] & maskOrZ;
   maskOrZero [ 7] = mask [ 7] & maskOrZ;
   maskOrZero [ 8] = mask [ 8] & maskOrZ;
   maskOrZero [ 9] = mask [ 9] & maskOrZ;
   maskOrZero [10] = mask [10] & maskOrZ;
   maskOrZero [11] = mask [11] & maskOrZ;
   maskOrZero [12] = mask [12] & maskOrZ;
   maskOrZero [13] = mask [13] & maskOrZ;
   maskOrZero [14] = mask [14] & maskOrZ;
   maskOrZero [15] = mask [15] & maskOrZ;

   // shift the whole data left by one bit
   bitShiftLeft1024gpr (data, 1);

   // xor in mask if bit 1023 was set
   data [ 0] ^= maskOrZero [ 0];
   data [ 1] ^= maskOrZero [ 1];
   data [ 2] ^= maskOrZero [ 2];
   data [ 3] ^= maskOrZero [ 3];
   data [ 4] ^= maskOrZero [ 4];
   data [ 5] ^= maskOrZero [ 5];
   data [ 6] ^= maskOrZero [ 6];
   data [ 7] ^= maskOrZero [ 7];
   data [ 8] ^= maskOrZero [ 8];
   data [ 9] ^= maskOrZero [ 9];
   data [10] ^= maskOrZero [10];
   data [11] ^= maskOrZero [11];
   data [12] ^= maskOrZero [12];
   data [13] ^= maskOrZero [13];
   data [14] ^= maskOrZero [14];
   data [15] ^= maskOrZero [15];
   }

//----------------------------------------------------------------------------
//
// lfsr2048gpr - left shift galois type lfsr for 2048-bit data, general purpose register implementation
//
static void lfsr2048gpr (uint64_t *data, uint64_t *mask)
   {
   uint64_t maskOrZero [32], maskOrZ;

   // expand data bit 2047 into mask
   maskOrZ = (int64_t)data [31] >> 63;

   // zero the mask if data bit 2047 is zero
   maskOrZero [ 0] = mask [ 0] & maskOrZ;
   maskOrZero [ 1] = mask [ 1] & maskOrZ;
   maskOrZero [ 2] = mask [ 2] & maskOrZ;
   maskOrZero [ 3] = mask [ 3] & maskOrZ;
   maskOrZero [ 4] = mask [ 4] & maskOrZ;
   maskOrZero [ 5] = mask [ 5] & maskOrZ;
   maskOrZero [ 6] = mask [ 6] & maskOrZ;
   maskOrZero [ 7] = mask [ 7] & maskOrZ;
   maskOrZero [ 8] = mask [ 8] & maskOrZ;
   maskOrZero [ 9] = mask [ 9] & maskOrZ;
   maskOrZero [10] = mask [10] & maskOrZ;
   maskOrZero [11] = mask [11] & maskOrZ;
   maskOrZero [12] = mask [12] & maskOrZ;
   maskOrZero [13] = mask [13] & maskOrZ;
   maskOrZero [14] = mask [14] & maskOrZ;
   maskOrZero [15] = mask [15] & maskOrZ;
   maskOrZero [16] = mask [16] & maskOrZ;
   maskOrZero [17] = mask [17] & maskOrZ;
   maskOrZero [18] = mask [18] & maskOrZ;
   maskOrZero [19] = mask [19] & maskOrZ;
   maskOrZero [20] = mask [20] & maskOrZ;
   maskOrZero [21] = mask [21] & maskOrZ;
   maskOrZero [22] = mask [22] & maskOrZ;
   maskOrZero [23] = mask [23] & maskOrZ;
   maskOrZero [24] = mask [24] & maskOrZ;
   maskOrZero [25] = mask [25] & maskOrZ;
   maskOrZero [26] = mask [26] & maskOrZ;
   maskOrZero [27] = mask [27] & maskOrZ;
   maskOrZero [28] = mask [28] & maskOrZ;
   maskOrZero [29] = mask [29] & maskOrZ;
   maskOrZero [30] = mask [30] & maskOrZ;
   maskOrZero [31] = mask [31] & maskOrZ;

   // shift the whole data left by one bit
   bitShiftLeft2048gpr (data, 1);

   // xor in mask if bit 2047 was set
   data [ 0] ^= maskOrZero [ 0];
   data [ 1] ^= maskOrZero [ 1];
   data [ 2] ^= maskOrZero [ 2];
   data [ 3] ^= maskOrZero [ 3];
   data [ 4] ^= maskOrZero [ 4];
   data [ 5] ^= maskOrZero [ 5];
   data [ 6] ^= maskOrZero [ 6];
   data [ 7] ^= maskOrZero [ 7];
   data [ 8] ^= maskOrZero [ 8];
   data [ 9] ^= maskOrZero [ 9];
   data [10] ^= maskOrZero [10];
   data [11] ^= maskOrZero [11];
   data [12] ^= maskOrZero [12];
   data [13] ^= maskOrZero [13];
   data [14] ^= maskOrZero [14];
   data [15] ^= maskOrZero [15];
   data [16] ^= maskOrZero [16];
   data [17] ^= maskOrZero [17];
   data [18] ^= maskOrZero [18];
   data [19] ^= maskOrZero [19];
   data [20] ^= maskOrZero [20];
   data [21] ^= maskOrZero [21];
   data [22] ^= maskOrZero [22];
   data [23] ^= maskOrZero [23];
   data [24] ^= maskOrZero [24];
   data [25] ^= maskOrZero [25];
   data [26] ^= maskOrZero [26];
   data [27] ^= maskOrZero [27];
   data [28] ^= maskOrZero [28];
   data [29] ^= maskOrZero [29];
   data [30] ^= maskOrZero [30];
   data [31] ^= maskOrZero [31];
   }

//----------------------------------------------------------------------------
//
// lfsr4096gpr - left shift galois type lfsr for 4096-bit data, general purpose register implementation
//
static void lfsr4096gpr (uint64_t *data, uint64_t *mask)
   {
   uint64_t maskOrZero [64], maskOrZ;

   // expand data bit 4095 into mask
   maskOrZ = (int64_t)data [63] >> 63;

   // zero the mask if data bit 4095 is zero
   maskOrZero [ 0] = mask [ 0] & maskOrZ;
   maskOrZero [ 1] = mask [ 1] & maskOrZ;
   maskOrZero [ 2] = mask [ 2] & maskOrZ;
   maskOrZero [ 3] = mask [ 3] & maskOrZ;
   maskOrZero [ 4] = mask [ 4] & maskOrZ;
   maskOrZero [ 5] = mask [ 5] & maskOrZ;
   maskOrZero [ 6] = mask [ 6] & maskOrZ;
   maskOrZero [ 7] = mask [ 7] & maskOrZ;
   maskOrZero [ 8] = mask [ 8] & maskOrZ;
   maskOrZero [ 9] = mask [ 9] & maskOrZ;
   maskOrZero [10] = mask [10] & maskOrZ;
   maskOrZero [11] = mask [11] & maskOrZ;
   maskOrZero [12] = mask [12] & maskOrZ;
   maskOrZero [13] = mask [13] & maskOrZ;
   maskOrZero [14] = mask [14] & maskOrZ;
   maskOrZero [15] = mask [15] & maskOrZ;
   maskOrZero [16] = mask [16] & maskOrZ;
   maskOrZero [17] = mask [17] & maskOrZ;
   maskOrZero [18] = mask [18] & maskOrZ;
   maskOrZero [19] = mask [19] & maskOrZ;
   maskOrZero [20] = mask [20] & maskOrZ;
   maskOrZero [21] = mask [21] & maskOrZ;
   maskOrZero [22] = mask [22] & maskOrZ;
   maskOrZero [23] = mask [23] & maskOrZ;
   maskOrZero [24] = mask [24] & maskOrZ;
   maskOrZero [25] = mask [25] & maskOrZ;
   maskOrZero [26] = mask [26] & maskOrZ;
   maskOrZero [27] = mask [27] & maskOrZ;
   maskOrZero [28] = mask [28] & maskOrZ;
   maskOrZero [29] = mask [29] & maskOrZ;
   maskOrZero [30] = mask [30] & maskOrZ;
   maskOrZero [31] = mask [31] & maskOrZ;
   maskOrZero [32] = mask [32] & maskOrZ;
   maskOrZero [33] = mask [33] & maskOrZ;
   maskOrZero [34] = mask [34] & maskOrZ;
   maskOrZero [35] = mask [35] & maskOrZ;
   maskOrZero [36] = mask [36] & maskOrZ;
   maskOrZero [37] = mask [37] & maskOrZ;
   maskOrZero [38] = mask [38] & maskOrZ;
   maskOrZero [39] = mask [39] & maskOrZ;
   maskOrZero [40] = mask [40] & maskOrZ;
   maskOrZero [41] = mask [41] & maskOrZ;
   maskOrZero [42] = mask [42] & maskOrZ;
   maskOrZero [43] = mask [43] & maskOrZ;
   maskOrZero [44] = mask [44] & maskOrZ;
   maskOrZero [45] = mask [45] & maskOrZ;
   maskOrZero [46] = mask [46] & maskOrZ;
   maskOrZero [47] = mask [47] & maskOrZ;
   maskOrZero [48] = mask [48] & maskOrZ;
   maskOrZero [49] = mask [49] & maskOrZ;
   maskOrZero [50] = mask [50] & maskOrZ;
   maskOrZero [51] = mask [51] & maskOrZ;
   maskOrZero [52] = mask [52] & maskOrZ;
   maskOrZero [53] = mask [53] & maskOrZ;
   maskOrZero [54] = mask [54] & maskOrZ;
   maskOrZero [55] = mask [55] & maskOrZ;
   maskOrZero [56] = mask [56] & maskOrZ;
   maskOrZero [57] = mask [57] & maskOrZ;
   maskOrZero [58] = mask [58] & maskOrZ;
   maskOrZero [59] = mask [59] & maskOrZ;
   maskOrZero [60] = mask [60] & maskOrZ;
   maskOrZero [61] = mask [61] & maskOrZ;
   maskOrZero [62] = mask [62] & maskOrZ;
   maskOrZero [63] = mask [63] & maskOrZ;

   // shift the whole data left by one bit
   bitShiftLeft4096gpr (data, 1);

   // xor in mask if bit 4095 was set
   data [ 0] ^= maskOrZero [ 0];
   data [ 1] ^= maskOrZero [ 1];
   data [ 2] ^= maskOrZero [ 2];
   data [ 3] ^= maskOrZero [ 3];
   data [ 4] ^= maskOrZero [ 4];
   data [ 5] ^= maskOrZero [ 5];
   data [ 6] ^= maskOrZero [ 6];
   data [ 7] ^= maskOrZero [ 7];
   data [ 8] ^= maskOrZero [ 8];
   data [ 9] ^= maskOrZero [ 9];
   data [10] ^= maskOrZero [10];
   data [11] ^= maskOrZero [11];
   data [12] ^= maskOrZero [12];
   data [13] ^= maskOrZero [13];
   data [14] ^= maskOrZero [14];
   data [15] ^= maskOrZero [15];
   data [16] ^= maskOrZero [16];
   data [17] ^= maskOrZero [17];
   data [18] ^= maskOrZero [18];
   data [19] ^= maskOrZero [19];
   data [20] ^= maskOrZero [20];
   data [21] ^= maskOrZero [21];
   data [22] ^= maskOrZero [22];
   data [23] ^= maskOrZero [23];
   data [24] ^= maskOrZero [24];
   data [25] ^= maskOrZero [25];
   data [26] ^= maskOrZero [26];
   data [27] ^= maskOrZero [27];
   data [28] ^= maskOrZero [28];
   data [29] ^= maskOrZero [29];
   data [30] ^= maskOrZero [30];
   data [31] ^= maskOrZero [31];
   data [32] ^= maskOrZero [32];
   data [33] ^= maskOrZero [33];
   data [34] ^= maskOrZero [34];
   data [35] ^= maskOrZero [35];
   data [36] ^= maskOrZero [36];
   data [37] ^= maskOrZero [37];
   data [38] ^= maskOrZero [38];
   data [39] ^= maskOrZero [39];
   data [40] ^= maskOrZero [40];
   data [41] ^= maskOrZero [41];
   data [42] ^= maskOrZero [42];
   data [43] ^= maskOrZero [43];
   data [44] ^= maskOrZero [44];
   data [45] ^= maskOrZero [45];
   data [46] ^= maskOrZero [46];
   data [47] ^= maskOrZero [47];
   data [48] ^= maskOrZero [48];
   data [49] ^= maskOrZero [49];
   data [50] ^= maskOrZero [50];
   data [51] ^= maskOrZero [51];
   data [52] ^= maskOrZero [52];
   data [53] ^= maskOrZero [53];
   data [54] ^= maskOrZero [54];
   data [55] ^= maskOrZero [55];
   data [56] ^= maskOrZero [56];
   data [57] ^= maskOrZero [57];
   data [58] ^= maskOrZero [58];
   data [59] ^= maskOrZero [59];
   data [60] ^= maskOrZero [60];
   data [61] ^= maskOrZero [61];
   data [62] ^= maskOrZero [62];
   data [63] ^= maskOrZero [63];
   }

//===========================================================================
//
// measure execution time and verify proper operation for each lfsr function
//

void lfsrGpr (TEST_DATA *testData, int loops, uint64_t cpuSpeed)
   {
   int index;
   uint64_t startCount;
   uint32_t pattern32, mask32;
   static uint64_t pattern64, pattern128 [2], pattern256 [4], pattern512 [8], pattern1024 [16], pattern2048 [32], pattern4096 [64];
   static uint64_t mask64   , mask128    [2], mask256    [4], mask512    [8], mask1024    [16], mask2048    [32], mask4096    [64];
  
   memcpy (&pattern32 , testData->initial , sizeof pattern32  );
   memcpy (&pattern64 , testData->initial , sizeof pattern64  );
   memcpy (pattern128 , testData->initial , sizeof pattern128 );
   memcpy (pattern256 , testData->initial , sizeof pattern256 );
   memcpy (pattern512 , testData->initial , sizeof pattern512 );
   memcpy (pattern1024, testData->initial , sizeof pattern1024);
   memcpy (pattern2048, testData->initial , sizeof pattern2048);
   memcpy (pattern4096, testData->initial , sizeof pattern4096);
   memcpy (&mask32    , testData->mask32  , sizeof mask32     );
   memcpy (&mask64    , testData->mask64  , sizeof mask64     );
   memcpy (mask128    , testData->mask128 , sizeof mask128    );
   memcpy (mask256    , testData->mask256 , sizeof mask256    );
   memcpy (mask512    , testData->mask512 , sizeof mask512    );
   memcpy (mask1024   , testData->mask1024, sizeof mask1024   );
   memcpy (mask2048   , testData->mask2048, sizeof mask2048   );
   memcpy (mask4096   , testData->mask4096, sizeof mask4096   );

   //----------------------------------------------------------------------------

   startCount = queryPerformanceCounter ();
   for (index = 0; index < loops; index++) pattern32 = lfsr32gpr (pattern32, mask32);

   printf ("lfsr32gpr   %s\n", cyclesText (queryPerformanceCounter () - startCount, cpuSpeed, loops));
   if (pattern32 != *testData->expected32) printf ("-----lfsr32gpr test failed-----\n");

   //----------------------------------------------------------------------------

   startCount = queryPerformanceCounter ();
   for (index = 0; index < loops; index++) pattern64 = lfsr64gpr (pattern64, mask64);

   printf ("lfsr64gpr   %s\n", cyclesText (queryPerformanceCounter () - startCount, cpuSpeed, loops));
   if (pattern64 != *testData->expected64) printf ("-----lfsr64gpr test failed-----\n");

   //----------------------------------------------------------------------------

   startCount = queryPerformanceCounter ();
   for (index = 0; index < loops; index++) lfsr128gpr (pattern128, mask128);

   printf ("lfsr128gpr  %s\n", cyclesText (queryPerformanceCounter () - startCount, cpuSpeed, loops));
   if (memcmp (pattern128, testData->expected128, sizeof pattern128) != 0) printf ("-----lfsr128gpr test failed-----\n");

   //----------------------------------------------------------------------------

   startCount = queryPerformanceCounter ();
   for (index = 0; index < loops; index++) lfsr256gpr (pattern256, mask256);

   printf ("lfsr256gpr  %s\n", cyclesText (queryPerformanceCounter () - startCount, cpuSpeed, loops));
   if (memcmp (pattern256, testData->expected256, sizeof pattern256) != 0) printf ("-----lfsr256gpr test failed-----\n");

   //----------------------------------------------------------------------------

   startCount = queryPerformanceCounter ();
   for (index = 0; index < loops; index++) lfsr512gpr (pattern512, mask512);

   printf ("lfsr512gpr  %s\n", cyclesText (queryPerformanceCounter () - startCount, cpuSpeed, loops));
   if (memcmp (pattern512, testData->expected512, sizeof pattern512) != 0) printf ("-----lfsr512gpr test failed-----\n");

   //----------------------------------------------------------------------------

   startCount = queryPerformanceCounter ();
   for (index = 0; index < loops; index++) lfsr1024gpr (pattern1024, mask1024);

   printf ("lfsr1024gpr %s\n", cyclesText (queryPerformanceCounter () - startCount, cpuSpeed, loops));
   if (memcmp (pattern1024, testData->expected1024, sizeof pattern1024) != 0) printf ("-----lfsr1024gpr test failed-----\n");

   //----------------------------------------------------------------------------

   startCount = queryPerformanceCounter ();
   for (index = 0; index < loops; index++) lfsr2048gpr (pattern2048, mask2048);

   printf ("lfsr2048gpr %s\n", cyclesText (queryPerformanceCounter () - startCount, cpuSpeed, loops));
   if (memcmp (pattern2048, testData->expected2048, sizeof pattern2048) != 0) printf ("-----lfsr2048gpr test failed-----\n");

   //----------------------------------------------------------------------------

   startCount = queryPerformanceCounter ();
   for (index = 0; index < loops; index++) lfsr4096gpr (pattern4096, mask4096);

   printf ("lfsr4096gpr %s\n", cyclesText (queryPerformanceCounter () - startCount, cpuSpeed, loops));
   if (memcmp (pattern4096, testData->expected4096, sizeof pattern4096) != 0) printf ("-----lfsr4096gpr test failed-----\n");

   //----------------------------------------------------------------------------
   }
